package net.sf.jsptest.assertion;

import java.util.List;

import junit.framework.Assert;
import net.sf.jsptest.utils.XML;

import org.jaxen.JaxenException;
import org.jaxen.dom.DOMXPath;
import org.w3c.dom.Element;

/**
 * @author Lasse Koskela
 */
public abstract class DOMAssertion extends AbstractAssertion {

    protected Element context;

    public Element getElement() {
        return context;
    }

    public void shouldContain(String text) {
        assertContains(context.getTextContent(), text);
    }

    public void shouldNotContain(String text) {
        assertDoesNotContain(context.getTextContent(), text);
    }

    public void shouldContainElement(String xpathExpression) {
        shouldContainElement("No matching nodes found for XPath: "
                + xpathExpression + " from\n" + getContextAsString(),
                xpathExpression);
    }

    public void shouldContainElement(String message,
            String xpathExpression) {
        try {
            DOMXPath xpath = new DOMXPath(xpathExpression);
            List matchingNodes = xpath.selectNodes(context);
            Assert.assertFalse(message, matchingNodes.isEmpty());
        } catch (JaxenException e) {
            throw new RuntimeException(e);
        }
    }

    protected String getContextAsString() {
        return XML.toString(context);
    }
}
