package net.sf.jsptest.assertion;

import net.sf.jsptest.HtmlTestCase;
import junit.framework.AssertionFailedError;

/**
 * @author Lasse Koskela
 */
public abstract class ExpectedAssertionFailure {

    private HtmlTestCase testcase;

    /**
     * Override this method to perform a test that should fail with an
     * exception.
     */
    protected abstract void run() throws Exception;

    /**
     * @param testcase
     *                The <tt>HtmlTestCase</tt> that serves as the context for
     *                this operation.
     */
    public ExpectedAssertionFailure(HtmlTestCase testcase)
            throws Exception {
        this(testcase, "Operation should've failed but it didn't.");
    }

    /**
     * @param testcase
     *                The <tt>HtmlTestCase</tt> that serves as the context for
     *                this operation.
     * @param message
     *                The message to display if the specified operation doesn't
     *                throw an AssertionFailedError.
     */
    public ExpectedAssertionFailure(HtmlTestCase testcase,
            String message) throws Exception {
        this.testcase = testcase;
        verify(message);
    }

    protected PageAssertion page() {
        return testcase.page();
    }

    protected OutputAssertion output() {
        return testcase.output();
    }

    protected ElementAssertion element(String xpath) {
        return testcase.element(xpath);
    }

    private void verify(String message) {
        try {
            run();
            throw new NoExceptionWasThrown();
        } catch (AssertionFailedError expected) {
            // everything went according to the plan!
        } catch (NoExceptionWasThrown e) {
            throw new AssertionFailedError(message);
        } catch (Throwable e) {
            throw new IncorrectExceptionError(
                    "A non-assertion exception was thrown: "
                            + e.getClass().getName(), e);
        }
    }

    public static class IncorrectExceptionError extends
            RuntimeException {
        public IncorrectExceptionError(String message, Throwable e) {
            super(message, e);
        }
    }

    private static class NoExceptionWasThrown extends Exception {
    }
}
