package net.sf.jsptest.assertion;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import junit.framework.Assert;

import net.sf.jsptest.html.Form;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * @author Lasse Koskela
 */
public class FormAssertion {

    private final List forms;

    public FormAssertion(Document document, ElementChooser chooser) {
        this.forms = new ArrayList();
        NodeList elements = document.getElementsByTagName("FORM");
        for (int i = 0; i < elements.getLength(); i++) {
            Element element = (Element) elements.item(i);
            if (chooser.accept(element)) {
                forms.add(new Form(element));
            }
        }
        if (forms.isEmpty()) {
            Assert.fail("Form not found.");
        }
    }

    public void shouldHaveField(String name) {
        for (Iterator i = forms.iterator(); i.hasNext();) {
            Form form = (Form) i.next();
            if (form.hasInputField(name)) {
                return;
            }
        }
        Assert.fail("No form field '" + name + "' on page");
    }

    public void shouldHaveSubmitButton(String nameOrLabel) {
        for (Iterator i = forms.iterator(); i.hasNext();) {
            Form form = (Form) i.next();
            if (form.hasSubmitButton(nameOrLabel)) {
                return;
            }
        }
        Assert.fail("No form submit button '" + nameOrLabel
                + "' on page");
    }

    public void shouldHaveSubmitButton() {
        for (Iterator i = forms.iterator(); i.hasNext();) {
            Form form = (Form) i.next();
            if (form.hasSubmitButton()) {
                return;
            }
        }
        Assert.fail("No form submit button on page");
    }

    public FormFieldAssertion field(String fieldName) {
        shouldHaveField(fieldName);
        return new FormFieldAssertion(forms, fieldName);
    }
}
